import logging

from flask import (
    Blueprint,
    Response,
    jsonify,
    request,
)


def init_route(app, server, url_prefix):
    bp = Blueprint("lims", __name__, url_prefix=url_prefix)

    @bp.route("/lims_samples", methods=["GET"])
    @server.restrict
    def get_proposal_samples():
        try:
            lims_name = request.args.get("lims")
            res = jsonify(app.lims.get_lims_samples(lims_name))
        except Exception:
            logging.getLogger("MX3.HWR").exception("Could not get Lims samples")
            res = (
                "Could not get Lims samples",
                409,
                {
                    "Content-Type": "application/json",
                },
            )

        return res

    @bp.route("/proposal", methods=["POST"])
    @server.restrict
    def set_proposal():
        """
        Set the selected proposal.
        """
        # proposal_number is the session identifier
        session_id = request.get_json().get("proposal_number", None)
        app.lims.select_session(session_id)
        app.usermanager.update_active_users()

        return Response(status=200)

    @bp.route("/proposal", methods=["GET"])
    @server.restrict
    def get_proposal():
        """
        Return the currently selected proposal.
        (The proposal list is part of the login_res)
        """
        return jsonify({"Proposal": app.lims.get_proposal_info()})

    return bp



import logging
from typing import ClassVar

# This needs to be a direct import of DataPublisher otherwise
# the is instance check fails due to different "import paths" It
# works because mxcubecore adds mxcubecore.HardwareObjects to
# sys path in __init__.py
import DataPublisher
from mxcubecore.BaseHardwareObjects import HardwareObjectState

from mxcubeweb.core.adapter.adapter_base import AdapterBase


class DataPublisherAdapter(AdapterBase):
    ATTRIBUTES = ["current_data", "all_data", "current"]
    SUPPORTED_TYPES: ClassVar[list[object]] = [DataPublisher.DataPublisher]

    def __init__(self, ho, *args):
        """
        Args:
            (object): Hardware object.
        """
        super().__init__(ho, *args)
        self._all_data_list = []
        self._current_data_list = []
        self._current_info = {}

        try:
            ho.connect("data", self._new_data_handler)
            ho.connect("start", self._start_handler)
            ho.connect("end", self._end_handler)
        except Exception:
            msg = "Could not initialize DataPublisherAdapter"
            logging.getLogger("MX3.HWR").exception(msg)
        else:
            self._available = True

    def _new_data_handler(self, data):
        self._current_data_list.append(data["data"])
        self.emit_ho_attribute_changed(
            "current_data", [data["data"]], operation="UPDATE"
        )

    def _start_handler(self, data):
        self._current_info = data
        self.emit_ho_changed()

    def _end_handler(self, data):
        self._all_data_list.append(data)
        self.emit_ho_changed()

    def state(self):
        return HardwareObjectState.READY.name

    def current_data(self) -> list:
        return self._current_data_list

    def current(self) -> dict:
        return self._current_info

    def all_data(self) -> list:
        return self._all_data_list



import importlib
import logging

from mxcubeweb.core.server.resource_handler import (
    ResourceHandlerFactory,
)


class ComponentBase:
    def __init__(self, app, config, resource_handler_config=None):
        self.app = app
        self.config = config

        if resource_handler_config:
            cls_name = self.__class__.__name__.lower()

            ResourceHandlerFactory.create_or_get(
                name=cls_name,
                url_prefix="/mxcube/api/v0.1/" + cls_name,
                handler_dict={cls_name: self},
                app=self.app,
                exports=resource_handler_config.exports,
                commands=resource_handler_config.commands,
                attributes=resource_handler_config.attributes,
                handler_type="component",
            )


def import_component(config, package="", module=""):
    _module = "mxcubeweb.core"

    if package:
        _module = f"{_module}.{package}"

    if not module:
        module = config.class_name.lower()

    _module = f"{_module}.{module}"
    mod = importlib.import_module(_module)
    _cls = getattr(mod, config.class_name)

    logging.getLogger("MX3").info(
        f"Using UserManager {_cls.__module__}.{_cls.__name__}"
    )

    return _cls
